%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main file for Simulation 2 (extended object) for the TIE tutorial.
% Version 2.0 -
% Related Reference:
% "Transport-of-intensity equationA tutoral"
% Chao Zuo, Jiaji Li, Jiasong Sun,Yao Fan, Jialin Zhang, Linpeng Lu, Runnan Zhang,
% Bowen Wang, Lei Huang, Qian Chen
% last modified on 02/27/2021
% by Chao Zuo (zuochao@njust.edu.cn,surpasszuo@163.com)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Clear everything existing.
clc
close all
clearvars
addpath('.\Functions');
addpath('.\Data\Logo')

%% Define simulation conditions
% Case 1: Uniform intensity (all 0.5);
% Case 2: Nonuniform intensity (no small intensity values)
% Case 3: Nonuniform intensity (with small intensity values)
% Case 4: Nonuniform intensity (with small intensity zeros)
Case = 2;

%% Read raw images for simulations
Phi = double((imread('TIE_Multiple_Blur2.tif')))/255;

if Case == 1
    % Case 1: Uniform intensity (all 0.5);
    I0 = ones(size(Phi))/2;
    JudgeFlag_DCT = 1;
    JudgeFlag_US = 1;
    MaxIterNum = 500;
elseif Case == 2
    % Case 2: Nonuniform intensity (no small intensity values)
    I0 = mat2gray(double((imread('SCILab_Blur.tif')))/255,[-0.2,1]);
    JudgeFlag_DCT = 1;
    JudgeFlag_US = 1;
    MaxIterNum = 500;
elseif Case == 3
    % Case 3: Nonuniform intensity (with small intensity values)
    I0 = mat2gray(double((imread('SCILab_Blur.tif')))/255,[0,1]);
    JudgeFlag_DCT = 0;
    JudgeFlag_US = 1;
    MaxIterNum = 50;
elseif Case == 4
    % Case 4: Nonuniform intensity (with small intensity zeros)
    I0 = mat2gray(double((imread('SCILab_Blur.tif'))));
    JudgeFlag_DCT = 0;
    JudgeFlag_US = 1;
    MaxIterNum = 500;
end



%% Define aperture for inhomogeneous BC
Aperture = false(size(Phi));
Aperture_size = 20;
Aperture(1+Aperture_size:end-Aperture_size,1+Aperture_size:end-Aperture_size) = true;
I0(~Aperture) = 0;

%% Defined all the parameters
% Set the pixel resolution of image.
[Ny,Nx] = size(Phi);

Pixelsize = 2e-6;   % pixel size (m)
Lambda = 0.633e-6;  % wavelength (m)
k = 2*pi/Lambda;    % wave number
dz = 1e-6;          % defocus distance (m)

%% Numerical propagation to generate defocused images.
% Complex amplitude of optical field in foucs.
U0 = sqrt(I0).*exp(1i.*Phi);
Phi(~Aperture) = NaN;
Phi(Aperture==0)=NaN;

% Numerical propagation for defocused images.
Method = 'Angular Spectrum';  %'TIE','Angular Spectrum','Fresnel'
[Uz,Iz] = Numerical_Propagation(U0,dz,Pixelsize,Lambda,Method);

% Axial intensity derivative.
dIdz = (Iz-I0)/dz;

% Check if satisfying Energy Conservation .
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    disp(['mean2(dIdz) = ' num2str(mean2(dIdz))]);
    warning('Conservation of Energy may not be satisfied!');
end

%% Show the images.
figure
subplot(1,2,1)
imshow(I0,[0,1]);
title('In-focus intensity');
colorbar
subplot(1,2,2)
imshow(Phi,[0,1]);
title('Phase');
colorbar

figure
subplot(1,3,1)
imshow(I0,[0,1]);
title('In-focus intensity');
subplot(1,3,2)
imshow(Iz,[0,1]);
title('Defocused intensity');
subplot(1,3,3)
imshow(dIdz,[]);
title('Intensity Derivative');
%% Solve TIE with iteration parameters.
RegPara = eps;
IntThr = 0.01;

%% Solve TIE with FFT-TIE.
[Phi_FFT,RMSE_FFT,Time_FFT] = TIE_FFT_solution(dIdz,I0,Pixelsize,k,RegPara,IntThr,Phi);
% Error Calculation.
phiErr_FFT = Remove_Piston(Phi_FFT - Phi);

%% Solve TIE with DCT-TIE.
[Phi_DCT,RMSE_DCT,Time_DCT] = TIE_DCT_solution(dIdz,I0,Pixelsize,k,RegPara,IntThr,Phi);
% Error Calculation.
phiErr_DCT = Remove_Piston(Phi_DCT - Phi);

%% Solve TIE with IterDCT-TIE.
[Phi_IterDCT,RMSE_IterDCT,Time_IterDCT] = Iter_DCT_solution(dIdz,dz,I0,Pixelsize,k,RegPara,MaxIterNum,JudgeFlag_DCT,Method,Phi);
% Error Calculation.
phiErr_IterDCT = Remove_Piston(Phi_IterDCT - Phi);

%% Solve TIE with US-TIE.
[Phi_US,RMSE_US,Time_US] = Universal_Solution(dIdz,dz,I0,Pixelsize,k,RegPara,MaxIterNum,JudgeFlag_US,Method,Phi);
% Error Calculation.
phiErr_US = Remove_Piston(Phi_US - Phi);


%% Show Reconstruction Result
figure;
scrsz = get(0,'ScreenSize');
set(gcf,'Position',scrsz);

subplot(2,4,1)
imshow(Phi_FFT,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (FFT-TIE)');
subplot(2,4,2)
imshow(Phi_DCT,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (DCT-TIE)');
subplot(2,4,3)
imshow(Phi_IterDCT,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (IterDCT-TIE)');
subplot(2,4,4)
imshow(Phi_US,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (US-TIE)');

subplot(2,4,5)
imshow(phiErr_FFT,[-0.05,0.05]);
colormap(gca,jet);colorbar
title(['Error (FFT-TIE) ','RMSE =',num2str(RMSE_FFT(end))]);
subplot(2,4,6)
imshow(phiErr_DCT,[-0.05,0.05]);
colormap(gca,jet);colorbar
title(['Error (DCT-TIE) ','RMSE = ', num2str(RMSE_DCT(end))]);
subplot(2,4,7)
imshow(phiErr_IterDCT,[-0.05,0.05]);
colormap(gca,jet);colorbar
title(['Error (IterDCT-TIE) ','RMSE = ', num2str(RMSE_IterDCT(end))]);
subplot(2,4,8)
imshow(phiErr_US,[-0.05,0.05]);
colormap(gca,jet);colorbar
title(['Error (US-TIE) ','RMSE = ', num2str(RMSE_US(end))]);

figure
plot(0:size(RMSE_US,2)-1,RMSE_US,'color',[0.8,0.2,0.2],'LineWidth',3);
hold on
plot(0:size(RMSE_IterDCT,2)-1,RMSE_IterDCT,'color',[0.2,0.2,0.8],'LineWidth',3);
title('RMSE Comparison')
ylim([0,0.3]);
legend('US-TIE','IterDCT');
xlabel('Iteration NO.')

%% Computation Time
figure
subplot(1,2,1)
plot(Time_IterDCT,'color',[0.8,0.2,0.2],'LineWidth',3);
hold on
plot(Time_US,'color',[0.2,0.2,0.8],'LineWidth',3);
title('Time per iteration (s)')
legend('IterDCT','US-TIE');
xlabel('Iteration NO.')
subplot(1,2,2)
TimeUS_plot=cumsum(Time_US);
Time_IterDCT_plot=cumsum(Time_IterDCT);
plot(Time_IterDCT_plot,'color',[0.8,0.2,0.2],'LineWidth',3);
hold on
plot(TimeUS_plot,'color',[0.2,0.2,0.8],'LineWidth',3);
title('Overall time (s)')
legend('IterDCT','US-TIE');
xlabel('Iteration NO.')
disp(['FFT-TIE computation time:', num2str(Time_FFT(end)),'s']);
disp(['DCT-TIE computation time:', num2str(Time_DCT(end)),'s']);
disp(['IterDCT computation time:', num2str(Time_IterDCT_plot(end)),'s']);
disp(['US--TIE computation time:', num2str(TimeUS_plot(end)),'s']);


